import { minAspectRatio, minWidth, minHeight, maxWidth } from "./base";
const orderByMin = [minWidth, minAspectRatio, minHeight];
const toOrderedBreakpointNames = (values, breakpoints, matchFunction) => {
  const result = Object.keys(values)
    .filter((e) => e !== "default")
    .sort((a, b) => breakpoints[a] - breakpoints[b]);
  if (orderByMin.includes(matchFunction)) {
    return result.slice().reverse();
  }
  return result;
};
/*
 * To debug use analog on CSS: https://codepen.io/retyui/pen/dyOzKzV
 */
export function createBreakpointsMatcher(
  breakpoints,
  matchFunction = minWidth
) {
  return function breakpointsMatcher(values) {
    /* istanbul ignore next */
    // @ts-expect-error: was removed
    if (process.env.NODE_ENV !== "production") {
      const invalidKeys = Object.keys(values).filter((key) => {
        return key !== "default" && breakpoints[key] === undefined;
      });
      if (invalidKeys.length > 0) {
        console.warn(`[react-native-stylex]: Invalid values was passed to 'breakpointsMatcher' function

allowed keys: ${Object.keys(breakpoints).join(", ")}
unexpected keys: ${invalidKeys.join(", ")}
`);
      }
    }
    const orderedBreakpointNames = toOrderedBreakpointNames(
      values,
      breakpoints,
      matchFunction
    );
    const key =
      orderedBreakpointNames.find((breakpointName) =>
        matchFunction(breakpoints[breakpointName], values[breakpointName])
      ) || "default";
    return values[key] || null;
  };
}
function getNextByKey(breakpoints, key) {
  const breakpointsKeys = Object.keys(breakpoints).sort(
    (a, b) => breakpoints[a] - breakpoints[b]
  );
  const index = breakpointsKeys.indexOf(key);
  const nextKey = breakpointsKeys[index + 1];
  return nextKey;
}
export function createBreakpoints(breakpoints) {
  function up(key, value) {
    return minWidth(breakpoints[key], value);
  }
  function down(key, value) {
    return maxWidth(breakpoints[key], value);
  }
  function only(key, value) {
    const nextKey = getNextByKey(breakpoints, key);
    if (nextKey !== undefined) {
      return minWidth(
        breakpoints[key],
        maxWidth(breakpoints[nextKey] - 0.05, value)
      );
    }
    return minWidth(breakpoints[key], value);
  }
  function between(start, end, value) {
    return minWidth(
      breakpoints[start],
      maxWidth(breakpoints[end] - 0.05, value)
    );
  }
  return { up, down, only, between };
}
